/* Copyright (c) 1997, MpegTV LLC */
#ifndef _mpeg_assert_h
#define _mpeg_assert_h
#ifndef lint
static const char rcsid_mpeg_assert_h[] = "$Header: /net/hopkins/var/home/tristan/mpeg/include/RCS/mpeg_assert.h,v 1.6 1998/05/29 22:48:36 tristan Exp $";
#endif

/*--------------------------------------------------------------------------+
| This file defines the following macros:									|
|																			|
| STMT(stuff): for macro that are statements								|
| abort_now(): abort with Internal error message							|
| assert(ex): abort if ex = 0												|
+--------------------------------------------------------------------------*/


#undef STMT

/* use STMT to implement macros with statement(s) */
#ifdef __GNUC__
	/* the following is more efficient with gcc */
#	define STMT(stuff)		(void)({stuff})
#else
	/* define ZERO to avoid lint warnings */
#	ifdef lint
#		define ZERO	abs(0)
#	else
#		define ZERO	0
#	endif
#	define STMT(stuff)		do { stuff } while(ZERO)
#endif


/*--------------------------------------------------------------------------+
| abort_now() prints the message "Internal error" with						|
| the name of the source file and the line where the error was detected		|
| and aborts.																|
+--------------------------------------------------------------------------*/

/* _abort_now() is an expression */
#define _abort_now()	( \
	fprintf(stderr, "\"%s\", line %d: Internal error\n", __FILE__, __LINE__), \
	abort(), \
	0)

#ifdef HP
/* abort_now() is a statement */
#define abort_now()	STMT( \
	fprintf(stderr, "\"%s\", line %d: Internal error\n", __FILE__, __LINE__); \
	exit(1); \
	)
#else
/* abort_now() is a statement */
#define abort_now()	STMT( \
	fprintf(stderr, "\"%s\", line %d: Internal error\n", __FILE__, __LINE__); \
	abort(); \
	)
#endif


/*--------------------------------------------------------------------------+
| assert(assertion) checks if assertion is true (not 0).					|
| If the assertion fails, prints the message "Internal error" with			|
| the name of the source file and the line where the error was detected		|
| and aborts.																|
|																			|
| Unless DEBUG is defined, assert(assertion) has no effect.					|
|																			|
| WARNING:																	|
| NO SIDE EFECT ALLOWED in the macro parameter!!!!!							|
+--------------------------------------------------------------------------*/

#undef assert

#ifdef DEBUG

#  define assert(ex)	STMT(if(!(ex)) abort_now();)

#else /* not DEBUG */

#if 0 /* Tristan Savatier - Mon Jan 13 21:46:16 PST 1997 */
//#	define assert(ex)	STMT()
#endif /* 0 - Tristan Savatier - Mon Jan 13 21:46:16 PST 1997 */

#	define assert(v)

#endif /* not DEBUG */

#if 0 /* Tristan Savatier - Fri Feb 14 20:44:14 PST 1997 */
///*--------------------------------------------------------------------------+
//| SHOULD_BE(assertion,string) checks if assertion is true (not 0).			|
//| If the assertion fails, a message											|
//| indicating the name of the source file, the line							|
//| number, the string, an error message according to the value of errno		|
//| and exit with status 1.													|
//+--------------------------------------------------------------------------*/
//
//#define SHOULD_BE(assertion,string)	\
//	STMT( \
//		if(!(assertion)) { \
//			fprintf(stderr,"\"%s\", line %d: ", __FILE__, __LINE__); \
//			perror(string); \
//			exit(EXIT_FAILURE); /*NOTREACHED*/ \
//		} \
//	)
#endif /* 0 - Tristan Savatier - Fri Feb 14 20:44:14 PST 1997 */

#define warning()	\
	STMT( \
 		fprintf(stderr, "\"%s\", line %d: Internal warning\n", __FILE__, __LINE__); \
	)

	
#endif	/* not _mpeg_assert_h */
