#ifndef lint
static const char rcsid[] = "$Header: /net/hopkins/home/tristan/mpeg/dec/RCS/simple_player.c,v 1.46 1999/04/15 10:19:12 tristan Exp $";
static const char copyright[] = "Copyright (c) 1997, MpegTV LLC";
#endif


/*
 * Disclaimer of Warranty
 *
 * These software programs are available to the user without any license fee or
 * royalty on an "as is" basis.  Copyright-holder disclaims
 * any and all warranties, whether express, implied, or statuary, including any
 * implied warranties or merchantability or of fitness for a particular
 * purpose.  In no event shall the copyright-holder be liable for any
 * incidental, punitive, or consequential damages of any kind whatsoever
 * arising from the use of these programs.
 *
 * This disclaimer of warranty extends to the user of these programs and user's
 * customers, employees, agents, transferees, successors, and assigns.
 *
 */

#include "mtvp/mtvp.h"

#include "forms.h"
#include "simple_player.h"
#include "simple_player_controls.h"

/*--------------------------------------------------------------------------+
| Configuration flags														|
+--------------------------------------------------------------------------*/
/*
 * Define TEST_SIH only to test the sample SIH (Stream Input Handler) plugin.
 * Value of TEST_SIH is path of the sample SIH dynamic library.
 * The sample SIH plugin handles URLs of the form "file:<filename>".
 * Try: $ simple_player file:random.mpg (assuming random.mpg is an MPEG
 * file located in the current directory).
 */
/*#define TEST_SIH	"../sih/sample_sih.so"*/

/*
 * Define TEST_VOH only to test the sample VOH (Video Output Handler) plugin.
 * Value of TEST_VOH is path of the sample VOH dynamic library.
 * The sample VOH plugin uses SDL to play full-screen video.
 */
/*#define TEST_VOH	"../voh/sample_voh.so"*/


/*--------------------------------------------------------------------------+
| Macros																	|
+--------------------------------------------------------------------------*/
/* for the msg.h package */
#define MSG_PTR (&Msg)

/*--------------------------------------------------------------------------+
| Data types																|
+--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------+
| Globals																	|
+--------------------------------------------------------------------------*/

/* The form pointers */
FD_controls *Controls;

/*--------------------------------------------------------------------------+
| Statics																	|
+--------------------------------------------------------------------------*/

/* to communicate with player */
static PMP_STRUCT	Player;

/* for the msg.h package */
static MSG_STRUCT Msg;

/*--------------------------------------------------------------------------+
| my_exit																|
+--------------------------------------------------------------------------*/
static void
my_exit(int n)
{	
	MESSAGE(("(pid=%u): exiting, status = %d", (u_int)getpid(), n));
	
	/* terminate (real exit this time) */
	exit(n);
}


/*--------------------------------------------------------------------------+
| at_close_controls															|
+--------------------------------------------------------------------------*/
static int 
at_close_controls(
	FL_FORM *fl_form,
	void	*data
)
{
	/*
	 * This callback is called when the controls form is closed.
	 */
	command(NULL, COM_QUIT);
	return FL_OK;
}

/*--------------------------------------------------------------------------+
| quit
+--------------------------------------------------------------------------*/
static int a2g_read_command(int fd);

static void 
quit()
{
	/* remove this callback (g2a_cleanup will close this pipe) */
	fl_remove_io_callback(Player.receive_channel.msg_pipe[PMP_FD_READ], FL_READ, (void (*)(int, void *))a2g_read_command);

	/* g2a_cleanup() should be called to close pipes to the (now defunct) player */
	g2a_cleanup(&Player);
	
	my_exit(EXIT_SUCCESS);
}

/*--------------------------------------------------------------------------+
| a2g_read_command															|
+--------------------------------------------------------------------------*/
static int
a2g_read_command(int fd)
{
	PMP_HEADER	msg;		/* to hold message header from Player */ 
	PMP_DATA_A2G	data;	/* to hold message data from Player */
	
	/*
	 * This  callback is called when there is something available for
	 * reading on Player.receive_channel.msg_pipe[PMP_FD_READ]
	 * (also equal to fd in this routine).
	 *
	 * This routine should read one message from the Player
	 * and take the appropriate action.
	 *
	 * It is mostly a place holder in this example: This sample
	 * application ignores most messages.  It just checks the
	 * API version number and quits if the message A2G_COM_QUIT
	 * is received (which should not happen, by the way!).
	 */
	 
	/* read message from player */
	if (a2g_read_msg(&Player, &msg, &data) == -1) {
		/* this can happen if the player is killed brutally */
		MESSAGE_ERROR(("a2g_read_msg: error reading msg"));
		quit();
	}

	/* trace message if (Msg_select & MSG_INFO) */
	{
		char buf[PMP_PRINT_MSG_BUF_SIZE];
		MESSAGE(("a2g_read_command: received msg: %s", pmp_print_msg(&msg,  (const char *)&data, buf)));
	}
	
	switch ((A2G_COM)msg.type) {
		case A2G_COM_STATE:
			/* show Player's state in the form title */
			switch (data.data_generic.data_int) {
			case STATE_STOPPED:
				fl_set_form_title(Controls->controls, "Simple Player (Stopped)");
				break;
			case STATE_PAUSED:
				fl_set_form_title(Controls->controls, "Simple Player (Paused)");
				break;
			case STATE_PLAYING:
				fl_set_form_title(Controls->controls, "Simple Player (Playing)");
				break;
			default:
				FATAL(("unknow State (should not happen!)"));
			}
			break;
#ifdef TEST_SIH
		case A2G_COM_SIH_FEEDBACK:
			{
				u_int payload_size = msg.size - sizeof(SIH_HANDLER_ID);
				const char *payload_data = (const char *)&data + sizeof(SIH_HANDLER_ID);
				int	handler_id = data.data_sih_handler_id;
				
				printf("received A2G_COM_SIH_FEEDBACK from handler_id = %d, size = %u, data = \"%s\"\n", handler_id, payload_size, payload_data);
			}
			break;
#endif /* TEST_SIH */
		case A2G_COM_RETURN_FAILURE:
			/*
			 * This message is received each time a command that was sent with
			 * the PMP_FLAG_RETURN_STATUS flag failed.
			 *
			 * This is convenient to test the return status of commands like
			 * COM_SIH_LINK or COM_OPEN_STREAM_URL.
			 *
			 * Note that if PMP_FLAG_RETURN_ORIGINAL
			 * was set too, a copy of the original message is sent back right
			 * after the A2G_COM_RETURN_FAILURE message.
			 *
			 * Here, in this example, we know that the only command sent with
			 * PMP_FLAG_RETURN_STATUS is COM_SIH_LINK. Ideally we should check
			 * what was the command associated with this returned status message.
			 */
			errno = data.data_return_failure.error;
#ifdef TEST_SIH
			if (data.data_return_success.original_header.type == COM_SIH_LINK) {
				FATAL_ERROR(("cannot link SIH plugin"));
			}
#endif /* TEST_SIH */
#ifdef TEST_VOH
			if (data.data_return_success.original_header.type == COM_VOH_LINK) {
				FATAL_ERROR(("cannot link VOH plugin"));
			}
			if (data.data_return_success.original_header.type == COM_VOH_SELECT) {
				FATAL_ERROR(("cannot select VOH plugin"));
			}
#endif /* TEST_VOH */
			break;
		case A2G_COM_RETURN_SUCCESS:
			/*
			 * This message is received each time a command that was sent with
			 * the PMP_FLAG_RETURN_STATUS flag succeded.
			 */
#ifdef TEST_SIH
			if (data.data_return_success.original_header.type == COM_SIH_LINK) {
				int	handler_id = data.data_return_success.value;
				printf("SIH plugin linked, handler_id = %d\n", handler_id);
				{
					struct {
						SIH_HANDLER_ID	sih_handler_id;
						char buf[256];
					} sih_cntl_data;
					
					sih_cntl_data.sih_handler_id = handler_id;
					strcpy(sih_cntl_data.buf, "testing COM_SIH_CNTL");
					g2a_send_custom(&Player, COM_SIH_CNTL, 0, (const void *)&sih_cntl_data, sizeof(SIH_HANDLER_ID) + strlen(sih_cntl_data.buf) + 1);
				}
			}
#endif /* TEST_SIH */
#ifdef TEST_VOH
			if (data.data_return_success.original_header.type == COM_VOH_LINK) {
				int	handler_id = data.data_return_success.value;
				printf("VOH plugin linked successfully, handler_id = %d\n", handler_id);
				g2a_send_int(&Player, COM_VOH_SELECT, PMP_FLAG_RETURN_STATUS, handler_id);
				{
					struct {
						int	voh_handler_id;
						char buf[256];
					} voh_cntl_data;
					
					voh_cntl_data.voh_handler_id = handler_id;
					strcpy(voh_cntl_data.buf, "testing COM_VOH_CNTL");
					g2a_send_custom(&Player, COM_VOH_CNTL, 0, (const void *)&voh_cntl_data, sizeof(int) + strlen(voh_cntl_data.buf) + 1);
				}
			}
			if (data.data_return_success.original_header.type == COM_VOH_SELECT) {
				printf("VOH plugin selected\n");
			}
#endif /* TEST_VOH */
			break;
		case A2G_COM_QUIT:
			/*
			 * mtvp sends A2G_COM_QUIT after receiving COM_QUIT and before exiting.
			 * mtv will terminate when it receives the SIGCHLD just after mtvp has exited.
			 */
			quit();

			break;
		default:
			/* msgs not understood should be ignored */
			break;
	}

	return 0;
}



/*--------------------------------------------------------------------------+
| command																	|
+--------------------------------------------------------------------------*/
void
command(
	FL_OBJECT *fl_object,
	long param
)
{
	COMMAND	com = param;
	
	/*
	 * This callback is called when clicking on any of the Xforms graphic controls
	 * of the control panel.  The param indicates which control was used.
	 */
	
	MESSAGE(("executing command call-back"));
	
	switch (com) {
		case ACTION_QUIT:
		
			/* tell Player to quit */
			g2a_send_void(&Player, COM_QUIT, 0);

			break;
		case ACTION_PLAY:
		
			/* tell Player to start playing */
			g2a_send_int(&Player, COM_STATE, 0, STATE_PLAYING);
			
			break;
		case ACTION_STEP:
		
			/* tell Player to Step one frame */
			/* Note: if player was playing, state changes automatically to STATE_PAUSED */
			g2a_send_void(&Player, COM_STEP, 0);
			
			break;
		case ACTION_REWIND:
		
			/* tell Player to Rewind */
			g2a_send_int(&Player, COM_SEEK, 0, 0);
			
			break;
		case ACTION_OPEN_FILE:
			{
				/* get file name with standard Xforms file selector */
				const char *url = fl_show_fselector ("Open",
										NULL,
										"*.mpg",
										NULL);
				if (url != NULL) {
			
					g2a_send_string(&Player, COM_OPEN_STREAM_URL, 0, url);
				}
			}
			break;
		case ACTION_OPEN_URL:
			{
				/* ask user to enter URL using Xforms gadget */
				const char *url = fl_show_input ("Enter URL:", "http://");

				if (url != NULL) {
			
					g2a_send_string(&Player, COM_OPEN_STREAM_URL, 0, url);
				}
			}
			break;
		default:
			/* this hould not happen! */
			FATAL_ERROR(("command callback does not expect command %d", com));
			break;
	}
}

/*--------------------------------------------------------------------------+
| init_form																	|
+--------------------------------------------------------------------------*/
static void
init_form(
	int		argc,
	const char **argv
)
{	
	if (fl_initialize (&argc, (char **)argv, "Simple Player (Stopped)", 0, 0) == NULL) {
		FATAL_ERROR(("cannot initialize xform"));
	}

	if ((Controls = create_form_controls()) == NULL) {
		FATAL_ERROR(("cannot create Controls form"));
	}
	
	fl_set_form_atclose(Controls->controls, at_close_controls, NULL);

	fl_show_form(Controls->controls, FL_PLACE_GEOMETRY, FL_FULLBORDER, "Simple Player");

	MESSAGE(("adding io call-back"));

	fl_add_io_callback(Player.receive_channel.msg_pipe[PMP_FD_READ], FL_READ, (void (*)(int, void *))a2g_read_command, NULL);

}



/*--------------------------------------------------------------------------+
| main																		|
+--------------------------------------------------------------------------*/
/*ARGSUSED*/
int
main(
	int argc,
	const char **argv,
	const char **envp
)
{
	const char	**p, **q;
	int		n;

	/*
	 * Set Msg.select == -1 to print all the warnings and messages.
	 * See msg.h for the MESSAGE() and WARNING() macros definitions.
	 */
	 
	/* Msg.select = -1; */

	MESSAGE(("started, pid = %u", (u_int)getpid()));
	
	/*
	 * Start the player
	 */
	
	{
		static const char *mtvp_argv[] = {
#ifdef HANDLE_CHALLENGE
			"mtvp",	/* name of the player (if no / in pathname, will search in PATH) */
#else
			"../../bin/mtvp",	/* name of the player (if no / in pathname, will search in PATH) */
#endif
			/* "-w", */	/* trace everything */
			/* "-a0", */	/* disable audio */
			NULL,	/* must be NULL-terminated */
		};

		/*
		 * First parameter of g2a_init() is a ptr to a NULL-terminated array
		 * of strings.
		 * The first string is the name of the Player ("mtvp").
		 * The following strings are possible parameters.
		 * If the first string is a full pathname (starting with "/")
		 * g2a_init() will run this program as "the Player".
		 *
		 * If the first string is not a full pathname (starting with "/")
		 * g2a_init() will look for the Player in the following places:
		 * - in the directory where argv[0] is located (if argv[0] contains a '/')
		 * - in the current directory (".")
		 * - in the PATH
		 *
		 * Typically first string is "mtvp", if the player was installed
		 * under this name.
		 *
		 * Second parameter of g2a_init() is a string that should be
		 * the argv[0] from the calling program.
		 */
	
		if (g2a_init(&Player, mtvp_argv, argv[0]) == -1) {
			if (errno == 0) {
				FATAL_ERROR(("cannot start player (%s) - probably mtvp API version mismatch", mtvp_argv[0]));
			} else if (errno == EPIPE) {
				FATAL_ERROR(("cannot start player (%s) - check that player is in your PATH!", mtvp_argv[0]));
			} else {
				FATAL_ERROR(("cannot start player (%s) - could not fork", mtvp_argv[0]));
			}
		}
	}
	
	/* initialize the form */
	init_form(argc, argv);
	
#ifdef TEST_SIH
	/* link custom stream interface */
	g2a_send_string(&Player, COM_SIH_LINK, PMP_FLAG_RETURN_STATUS, TEST_SIH);
#endif
	
#ifdef TEST_VOH
	/* link custom stream interface */
	g2a_send_string(&Player, COM_VOH_LINK, PMP_FLAG_RETURN_STATUS, TEST_VOH);
#endif
	
	/* if one argument was passed, assume that it is a URL to open */
	if (argc == 2) {

		/* open stream from URL */
		g2a_send_string(&Player, COM_OPEN_STREAM_URL, 0, argv[1]);
		
	} else if (argc > 2) {
		fprintf(stderr, "usage: simple_player [URL]\n");
		fprintf(stderr, "example: simple_player file:movie.mpg\n");
		my_exit(EXIT_FAILURE);
	}

	MESSAGE(("entering event loop"));

	/* enter event loop */
	fl_do_forms();
	
	MESSAGE(("What?!? fl_do_forms() returned! (should never happen)"));

	/* terminated successfully */
	my_exit(EXIT_SUCCESS);

	/* so that lint does not complain! */
	return 0;
}


/* eof */
